import pytest
from bs4 import BeautifulSoup
from seo_analyzer.mobile_analyzer import MobileFriendlinessAnalyzer

@pytest.fixture
def analyzer():
    return MobileFriendlinessAnalyzer()

@pytest.fixture
def sample_html():
    return """
    <html>
        <head>
            <meta name="viewport" content="width=device-width, initial-scale=1">
            <style>
                @media (max-width: 768px) {
                    body { font-size: 16px; }
                }
                .small-text { font-size: 10px; }
            </style>
        </head>
        <body>
            <img src="test.jpg" srcset="test-small.jpg 300w, test-large.jpg 900w">
            <img src="static.jpg" class="img-fluid">
            <img src="fixed.jpg">
            <button style="padding: 20px;">Large Button</button>
            <a href="#" style="padding: 5px;">Small Link</a>
            <p class="small-text">Small text content</p>
        </body>
    </html>
    """

@pytest.fixture
def soup(sample_html):
    return BeautifulSoup(sample_html, 'html.parser')

def test_check_viewport(analyzer, soup):
    analyzer._check_viewport(soup)
    assert analyzer.viewport_settings == 'width=device-width, initial-scale=1'

def test_analyze_responsive_images(analyzer, soup):
    analyzer._analyze_responsive_images(soup)
    assert analyzer.total_images == 3
    assert analyzer.responsive_images == 2  # One with srcset, one with img-fluid class

def test_is_fluid_image(analyzer):
    img_tag = BeautifulSoup('<img class="img-fluid" src="test.jpg">', 'html.parser').find('img')
    assert analyzer._is_fluid_image(img_tag) is True

    img_tag = BeautifulSoup('<img style="width: 100%;" src="test.jpg">', 'html.parser').find('img')
    assert analyzer._is_fluid_image(img_tag) is True

    img_tag = BeautifulSoup('<img src="test.jpg">', 'html.parser').find('img')
    assert analyzer._is_fluid_image(img_tag) is False

def test_analyze_media_queries(analyzer, soup):
    analyzer._analyze_media_queries(soup)
    assert len(analyzer.media_queries) > 0
    assert any('max-width: 768px' in query for query in analyzer.media_queries)

def test_analyze_touch_elements(analyzer, soup):
    analyzer._analyze_touch_elements(soup)
    assert analyzer.touch_elements == 2  # One button and one link

def test_analyze_font_sizes(analyzer, soup):
    analyzer._analyze_font_sizes(soup)
    assert len(analyzer.font_size_issues) == 1  # One element with 10px font

def test_extract_font_size(analyzer):
    element = BeautifulSoup('<p style="font-size: 12px;">Test</p>', 'html.parser').find('p')
    assert analyzer._extract_font_size(element) == '12px'

    element = BeautifulSoup('<p>Test</p>', 'html.parser').find('p')
    assert analyzer._extract_font_size(element) is None

def test_is_font_size_too_small(analyzer):
    assert analyzer._is_font_size_too_small('10px') is True
    assert analyzer._is_font_size_too_small('16px') is False
    assert analyzer._is_font_size_too_small('0.5em') is True
    assert analyzer._is_font_size_too_small('1em') is False
    assert analyzer._is_font_size_too_small('invalid') is False

def test_generate_results(analyzer, soup):
    analyzer.analyze(soup)
    results = analyzer._generate_results()
    
    assert isinstance(results, dict)
    assert 'viewport_meta' in results
    assert 'responsive_images' in results
    assert 'media_queries_found' in results
    assert 'touch_elements' in results
    assert 'font_size_issues' in results
    assert 'recommendations' in results

def test_generate_recommendations(analyzer, soup):
    # Setup some test conditions
    analyzer.viewport_settings = None
    analyzer.total_images = 4
    analyzer.responsive_images = 1
    analyzer.media_queries = []
    analyzer.touch_elements = 5
    analyzer.font_size_issues = [{'element': 'p', 'font_size': '10px'}]

    recommendations = analyzer._generate_recommendations()
    
    assert isinstance(recommendations, list)
    assert any('viewport' in rec.lower() for rec in recommendations)
    assert any('responsive' in rec.lower() for rec in recommendations)
    assert any('media queries' in rec.lower() for rec in recommendations)
    assert any('touch targets' in rec.lower() for rec in recommendations)
    assert any('font size' in rec.lower() for rec in recommendations)